// Copyright (c) 2003-2010 Christopher M. Kohlhoff
// Modifications (c) 2011 by Guilherme Maciel Ferreira / Intra2net AG
//
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)
#ifndef ICMPV4_HEADER_H
#define ICMPV4_HEADER_H

#include <stdint.h>

#include <istream>
#include <ostream>

#include <boost/shared_ptr.hpp>

#include "icmp/icmpmessage.h"
#include "icmp/icmptype.h"

//-----------------------------------------------------------------------------
// Icmpv4Header
//-----------------------------------------------------------------------------

/**
 * @brief This class represents the ICMP version 4 Header.
 *
 * The ICMP Generic Header Format is:
 *
 * @code
 * 0               8               16                             31
 * +---------------+---------------+------------------------------+     ---
 * |               |               |                              |      ^
 * |     type      |     code      |          checksum            |   4 bytes
 * |               |               |                              |      v
 * +---------------+---------------+------------------------------+     ---
 * |                                                              |
 * |                     specific to each message                 |
 * |                                                              |
 * +-------------------------------+------------------------------+
 * @endcode
 */
class Icmpv4Header
{
public:
    Icmpv4Header();
    Icmpv4Header(
            const Icmpv4Type type,
            const uint8_t code,
            const uint16_t checksum,
            const uint16_t identifier,
            const uint16_t sequence_number
    );

    Icmpv4Type get_type() const;
    void set_type( const Icmpv4Type type );

    uint8_t get_code() const;
    void set_code( const uint8_t code );

    uint16_t get_checksum() const;
    void set_checksum( const uint16_t checksum );

    uint16_t get_identifier() const;
    void set_identifier( const uint16_t identifier );

    uint16_t get_sequence_number() const;
    void set_sequence_number( const uint16_t sequence_number );

    boost::shared_ptr<IcmpMessage> get_icmp_message_format() const;
    void set_icmp_message_format( const Icmpv4Type type );
    void set_icmp_message_format( std::istream &is );

    size_t get_header_length() const;

    friend std::istream& operator>>(
            std::istream &is,
            Icmpv4Header &header
    );
    friend std::ostream& operator<<(
            std::ostream &os,
            const Icmpv4Header &header
    );

private:
    /// Changeable pointer to different ICMP messages types.
    boost::shared_ptr<IcmpMessage> MessageFormat;

};

#endif // ICMPV4_HEADER_H
